@@ -3,7 +3,6 @@ import logging
 import random
 
 import engineio
-import six
 
 from . import client
 from . import exceptions
@@ -37,12 +36,6 @@ class AsyncClient(client.Client):
                    use. To disable logging set to ``False``. The default is
                    ``False``. Note that fatal errors are logged even when
                    ``logger`` is ``False``.
-    :param binary: ``True`` to support binary payloads, ``False`` to treat all
-                   payloads as text. On Python 2, if this is set to ``True``,
-                   ``unicode`` values are treated as text, and ``str`` and
-                   ``bytes`` values are treated as binary.  This option has no
-                   effect on Python 3, where text and binary payloads are
-                   always automatically discovered.
     :param json: An alternative json module to use for encoding and decoding
                  packets. Custom json modules must have ``dumps`` and ``loads``
                  functions that are compatible with the standard library
@@ -77,16 +70,19 @@ class AsyncClient(client.Client):
                            are ``'polling'`` and ``'websocket'``. If not
                            given, the polling transport is connected first,
                            then an upgrade to websocket is attempted.
-        :param namespaces: The list of custom namespaces to connect, in
-                           addition to the default namespace. If not given,
-                           the namespace list is obtained from the registered
-                           event handlers.
+        :param namespaces: The namespaces to connect as a string or list of
+                           strings. If not given, the namespaces that have
+                           registered event handlers are connected.
         :param socketio_path: The endpoint where the Socket.IO server is
                               installed. The default value is appropriate for
                               most cases.
 
         Note: this method is a coroutine.
 
+        Note: The connection mechannism occurs in the background and will
+        complete at some point after this function returns. The connection
+        will be established when the ``connect`` event is invoked.
+
         Example usage::
 
             sio = socketio.AsyncClient()
@@ -99,18 +95,22 @@ class AsyncClient(client.Client):
         self.socketio_path = socketio_path
 
         if namespaces is None:
-            namespaces = set(self.handlers.keys()).union(
-                set(self.namespace_handlers.keys()))
-        elif isinstance(namespaces, six.string_types):
+            namespaces = list(set(self.handlers.keys()).union(
+                set(self.namespace_handlers.keys())))
+            if len(namespaces) == 0:
+                namespaces = ['/']
+        elif isinstance(namespaces, str):
             namespaces = [namespaces]
-            self.connection_namespaces = namespaces
-        self.namespaces = [n for n in namespaces if n != '/']
+        self.connection_namespaces = namespaces
         try:
             await self.eio.connect(url, headers=headers,
                                    transports=transports,
                                    engineio_path=socketio_path)
         except engineio.exceptions.ConnectionError as exc:
-            six.raise_from(exceptions.ConnectionError(exc.args[0]), None)
+            await self._trigger_event(
+                'connect_error', '/',
+                exc.args[1] if len(exc.args) > 1 else exc.args[0])
+            raise exceptions.ConnectionError(exc.args[0]) from None
         self.connected = True
 
     async def wait(self):
@@ -136,17 +136,17 @@ class AsyncClient(client.Client):
         :param event: The event name. It can be any string. The event names
                       ``'connect'``, ``'message'`` and ``'disconnect'`` are
                       reserved and should not be used.
-        :param data: The data to send to the client or clients. Data can be of
-                     type ``str``, ``bytes``, ``list`` or ``dict``. If a
-                     ``list`` or ``dict``, the data will be serialized as JSON.
+        :param data: The data to send to the server. Data can be of
+                     type ``str``, ``bytes``, ``list`` or ``dict``. To send
+                     multiple arguments, use a tuple where each element is of
+                     one of the types indicated above.
         :param namespace: The Socket.IO namespace for the event. If this
                           argument is omitted the event is emitted to the
                           default namespace.
         :param callback: If given, this function will be called to acknowledge
-                         the the client has received the message. The arguments
+                         the the server has received the message. The arguments
                          that will be passed to the function are those provided
-                         by the client. Callback functions can only be used
-                         when addressing an individual client.
+                         by the server.
 
         Note: this method is not designed to be used concurrently. If multiple
         tasks are emitting at the same time on the same client connection, then
@@ -157,7 +157,7 @@ class AsyncClient(client.Client):
         Note 2: this method is a coroutine.
         """
         namespace = namespace or '/'
-        if namespace != '/' and namespace not in self.namespaces:
+        if namespace not in self.namespaces:
             raise exceptions.BadNamespaceError(
                 namespace + ' is not a connected namespace.')
         self.logger.info('Emitting event "%s" [%s]', event, namespace)
@@ -165,10 +165,6 @@ class AsyncClient(client.Client):
             id = self._generate_ack_id(namespace, callback)
         else:
             id = None
-        if six.PY2 and not self.binary:
-            binary = False  # pragma: nocover
-        else:
-            binary = None
         # tuples are expanded to multiple arguments, everything else is sent
         # as a single argument
         if isinstance(data, tuple):
@@ -178,8 +174,7 @@ class AsyncClient(client.Client):
         else:
             data = []
         await self._send_packet(packet.Packet(
-            packet.EVENT, namespace=namespace, data=[event] + data, id=id,
-            binary=binary))
+            packet.EVENT, namespace=namespace, data=[event] + data, id=id))
 
     async def send(self, data, namespace=None, callback=None):
         """Send a message to one or more connected clients.
@@ -187,17 +182,17 @@ class AsyncClient(client.Client):
         This function emits an event with the name ``'message'``. Use
         :func:`emit` to issue custom event names.
 
-        :param data: The data to send to the client or clients. Data can be of
-                     type ``str``, ``bytes``, ``list`` or ``dict``. If a
-                     ``list`` or ``dict``, the data will be serialized as JSON.
+        :param data: The data to send to the server. Data can be of
+                     type ``str``, ``bytes``, ``list`` or ``dict``. To send
+                     multiple arguments, use a tuple where each element is of
+                     one of the types indicated above.
         :param namespace: The Socket.IO namespace for the event. If this
                           argument is omitted the event is emitted to the
                           default namespace.
         :param callback: If given, this function will be called to acknowledge
-                         the the client has received the message. The arguments
+                         the the server has received the message. The arguments
                          that will be passed to the function are those provided
-                         by the client. Callback functions can only be used
-                         when addressing an individual client.
+                         by the server.
 
         Note: this method is a coroutine.
         """
@@ -210,9 +205,10 @@ class AsyncClient(client.Client):
         :param event: The event name. It can be any string. The event names
                       ``'connect'``, ``'message'`` and ``'disconnect'`` are
                       reserved and should not be used.
-        :param data: The data to send to the client or clients. Data can be of
-                     type ``str``, ``bytes``, ``list`` or ``dict``. If a
-                     ``list`` or ``dict``, the data will be serialized as JSON.
+        :param data: The data to send to the server. Data can be of
+                     type ``str``, ``bytes``, ``list`` or ``dict``. To send
+                     multiple arguments, use a tuple where each element is of
+                     one of the types indicated above.
         :param namespace: The Socket.IO namespace for the event. If this
                           argument is omitted the event is emitted to the
                           default namespace.
@@ -240,7 +236,7 @@ class AsyncClient(client.Client):
         try:
             await asyncio.wait_for(callback_event.wait(), timeout)
         except asyncio.TimeoutError:
-            six.raise_from(exceptions.TimeoutError(), None)
+            raise exceptions.TimeoutError() from None
         return callback_args[0] if len(callback_args[0]) > 1 \
             else callback_args[0][0] if len(callback_args[0]) == 1 \
             else None
@@ -255,8 +251,6 @@ class AsyncClient(client.Client):
         for n in self.namespaces:
             await self._send_packet(packet.Packet(packet.DISCONNECT,
                                     namespace=n))
-        await self._send_packet(packet.Packet(
-            packet.DISCONNECT, namespace='/'))
         self.connected = False
         await self.eio.disconnect(abort=True)
 
@@ -293,37 +287,28 @@ class AsyncClient(client.Client):
         """Send a Socket.IO packet to the server."""
         encoded_packet = pkt.encode()
         if isinstance(encoded_packet, list):
-            binary = False
             for ep in encoded_packet:
-                await self.eio.send(ep, binary=binary)
-                binary = True
+                await self.eio.send(ep)
         else:
-            await self.eio.send(encoded_packet, binary=False)
+            await self.eio.send(encoded_packet)
 
-    async def _handle_connect(self, namespace):
+    async def _handle_connect(self, namespace, data):
         namespace = namespace or '/'
-        self.logger.info('Namespace {} is connected'.format(namespace))
-        await self._trigger_event('connect', namespace=namespace)
-        if namespace == '/':
-            for n in self.namespaces:
-                await self._send_packet(packet.Packet(packet.CONNECT,
-                                        namespace=n))
-        elif namespace not in self.namespaces:
-            self.namespaces.append(namespace)
+        if namespace not in self.namespaces:
+            self.logger.info('Namespace {} is connected'.format(namespace))
+            self.namespaces[namespace] = (data or {}).get('sid', self.sid)
+            await self._trigger_event('connect', namespace=namespace)
 
     async def _handle_disconnect(self, namespace):
         if not self.connected:
             return
         namespace = namespace or '/'
-        if namespace == '/':
-            for n in self.namespaces:
-                await self._trigger_event('disconnect', namespace=n)
-            self.namespaces = []
         await self._trigger_event('disconnect', namespace=namespace)
         if namespace in self.namespaces:
-            self.namespaces.remove(namespace)
-        if namespace == '/':
+            del self.namespaces[namespace]
+        if not self.namespaces:
             self.connected = False
+            await self.eio.disconnect(abort=True)
 
     async def _handle_event(self, namespace, id, data):
         namespace = namespace or '/'
@@ -338,13 +323,8 @@ class AsyncClient(client.Client):
                 data = list(r)
             else:
                 data = [r]
-            if six.PY2 and not self.binary:
-                binary = False  # pragma: nocover
-            else:
-                binary = None
             await self._send_packet(packet.Packet(
-                packet.ACK, namespace=namespace, id=id, data=data,
-                binary=binary))
+                packet.ACK, namespace=namespace, id=id, data=data))
 
     async def _handle_ack(self, namespace, id, data):
         namespace = namespace or '/'
@@ -373,9 +353,9 @@ class AsyncClient(client.Client):
             data = (data,)
         await self._trigger_event('connect_error', namespace, *data)
         if namespace in self.namespaces:
-            self.namespaces.remove(namespace)
+            del self.namespaces[namespace]
         if namespace == '/':
-            self.namespaces = []
+            self.namespaces = {}
             self.connected = False
 
     async def _trigger_event(self, event, namespace, *args):
@@ -436,10 +416,12 @@ class AsyncClient(client.Client):
                 break
         client.reconnecting_clients.remove(self)
 
-    def _handle_eio_connect(self):
+    async def _handle_eio_connect(self):
         """Handle the Engine.IO connection event."""
         self.logger.info('Engine.IO connection established')
         self.sid = self.eio.sid
+        for n in self.connection_namespaces:
+            await self._send_packet(packet.Packet(packet.CONNECT, namespace=n))
 
     async def _handle_eio_message(self, data):
         """Dispatch Engine.IO messages."""
@@ -454,7 +436,7 @@ class AsyncClient(client.Client):
         else:
             pkt = packet.Packet(encoded_packet=data)
             if pkt.packet_type == packet.CONNECT:
-                await self._handle_connect(pkt.namespace)
+                await self._handle_connect(pkt.namespace, pkt.data)
             elif pkt.packet_type == packet.DISCONNECT:
                 await self._handle_disconnect(pkt.namespace)
             elif pkt.packet_type == packet.EVENT:
@@ -464,7 +446,7 @@ class AsyncClient(client.Client):
             elif pkt.packet_type == packet.BINARY_EVENT or \
                     pkt.packet_type == packet.BINARY_ACK:
                 self._binary_packet = pkt
-            elif pkt.packet_type == packet.ERROR:
+            elif pkt.packet_type == packet.CONNECT_ERROR:
                 await self._handle_error(pkt.namespace, pkt.data)
             else:
                 raise ValueError('Unknown packet type.')
@@ -476,8 +458,7 @@ class AsyncClient(client.Client):
         if self.connected:
             for n in self.namespaces:
                 await self._trigger_event('disconnect', namespace=n)
-            await self._trigger_event('disconnect', namespace='/')
-            self.namespaces = []
+            self.namespaces = {}
             self.connected = False
         self.callbacks = {}
         self._binary_packet = None
