@@ -259,7 +259,8 @@ class AsyncServer(server.Server):
         the user session, use the ``session`` context manager instead.
         """
         namespace = namespace or '/'
-        eio_session = await self.eio.get_session(sid)
+        eio_sid = self.manager.eio_sid_from_sid(sid, namespace)
+        eio_session = await self.eio.get_session(eio_sid)
         return eio_session.setdefault(namespace, {})
 
     async def save_session(self, sid, session, namespace=None):
@@ -271,7 +272,8 @@ class AsyncServer(server.Server):
                           the default namespace is used.
         """
         namespace = namespace or '/'
-        eio_session = await self.eio.get_session(sid)
+        eio_sid = self.manager.eio_sid_from_sid(sid, namespace)
+        eio_session = await self.eio.get_session(eio_sid)
         eio_session[namespace] = session
 
     def session(self, sid, namespace=None):
